# Generic behavior for all tick counting/timing sources
module TickSource
	# Initialize the TickSource, must be called at the end
	# of the constructor
	def initSource
		# @dependents is the list of dependent objects
		@dependents = []
		@active = true
	end
	
	# Add a dependent to TickSource. Wheneven the TickSource
	# is updated, the <tt>update()</tt> method method of all dependents
	# is called with the current tick count as the only parameter
	def add(obj)
		@dependents << obj
	end
	
	# Get the current tick count from the TickSource
	def getTicks()
		@currentTick
	end
	
	# Send <tt>update()</tt> to all of the dependent objects
	def updateDependents()
		@dependents.each {|obj| obj.update(getTicks()) }
	end
	
	# Activates the TickSource.
	def start()
		@active = true
	end
	
	# Deactivates the TickSource. It will remain inactive until <tt>start()</tt> is
	# called, and will not send <tt>update()</tt> to dependents.
	def stop()
		@active = false
	end
end

# TickTimers keep a constant amount of time between ticks,
# calling a block of code if there is time before
# the tick ends.
class TickTimer
	include TickSource
	
	def initialize(tps)
		@initT = Time.now.to_f
		@step = 1.0/Float(tps)
		@currentTick = 0
		@lostTicks = 0
		initSource()
	end
	
	def to_seconds
		@currentTick*@step
	end
	
	# Returns a float value for the end of the tick.
	# Can be compared against <tt>Time.now.to_f()</tt>.
	def tickEndTime()
		(@currentTick + @lostTicks + 1)*@step + @initT
	end
	
	# Returns true if the there is time remaining in the current tick.
	def onTime?()
		Time.now.to_f < tickEndTime()
	end
	
	# Calls the associated block if the there is still time in the
	# current tick, and then sleeps until the end.
	def stepTick()
		yield() if(onTime?)
		t = tickEndTime() - Time.now.to_f
		sleep(t) if(t > 0)
		if(@active)
			updateDependents()
			@currentTick += 1
		else
			@lostTicks += 1
		end
	end
end

# TickSource that counts ticks from another TickSource, but can be activated
# and deactivated seperately from its parent.
class TickCounter
	include TickSource
	
	def initialize()
		@currentTick = 0
		@lostTicks = 0
	end
	
	def update(t)
		if(@active)
			updateDependents()
			@currentTick += 1
		end
	end
end